from django.contrib.auth.models import AbstractUser
from django.contrib.auth import get_user_model
from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

import ipaddress
import re

def validate_ula_prefix(prefix):
    ula_net = ipaddress.IPv6Network("fd00::/8")
    net_str = f"{prefix}/48"
    net = ipaddress.IPv6Network(net_str, strict=False)

    if not net.subnet_of(ula_net):
        raise ValidationError(
            _(f"Prefix {net} is not within ULA range ({ula_net})"),
            code='invalid',
            params = {'prefix': net }
        )


    try:
        ULA.objects.get(prefix=str(net[0]))
    except ULA.DoesNotExist:
        pass
    else:
        raise ValidationError(
            _(f"Prefix {net} is already registered"),
            code='invalid',
            params = {'prefix': net }
        )


class User(AbstractUser):
    pass

class ULA(models.Model):
    owner = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE
    )

    prefix = models.GenericIPAddressField(protocol='IPv6', unique=True, validators=[validate_ula_prefix])
    name = models.CharField(max_length=256)
    organization = models.CharField(max_length=256)
    website = models.URLField()

    def save(self, *args, **kwargs):
        """
        Ensure we only save the first IPv6 address of the network
        """

        net_str = f"{self.prefix}/48"
        net = ipaddress.IPv6Network(net_str, strict=False)
        self.prefix = str(net[0])

        super().save(*args, **kwargs)

    def __str__(self):
        return f"{self.prefix}"
